@@ -245,7 +245,7 @@ sparcPromGetBool(sbusPromNodePtr pnode, const char *prop)
     return promGetBool(prop);
 }
 
-static char *
+static const char *
 promWalkGetDriverName(int node, int oldnode)
 {
     int nextnode;
@@ -275,7 +275,7 @@ promWalkGetDriverName(int node, int oldnode)
 
     nextnode = promGetChild(node);
     if (nextnode) {
-        char *name;
+        const char *name;
 
         name = promWalkGetDriverName(nextnode, node);
         if (name)
@@ -288,10 +288,10 @@ promWalkGetDriverName(int node, int oldnode)
     return NULL;
 }
 
-char *
+const char *
 sparcDriverName(void)
 {
-    char *name;
+    const char *name;
 
     if (sparcPromInit() < 0)
         return NULL;
@@ -401,7 +401,7 @@ sparcPromAssignNodes(void)
         int fbNum, devId;
         static struct {
             int devId;
-            char *prefix;
+            const char *prefix;
         } procFbPrefixes[] = {
             {SBUS_DEVICE_BW2, "BWtwo"},
             {SBUS_DEVICE_CG14, "CGfourteen"},
